"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var ip = require("ip");
var url = require("url");
var requireModule_1 = require("./requireModule");
var pkg = requireModule_1.default('./package.json');
var mobileAssetTest = /\.(bmp|gif|jpg|jpeg|png|psd|svg|webp|m4v|aac|aiff|caf|m4a|mp3|wav|html|pdf|ttf)$/;
var babelUsed = false;
var useBabel = function () {
    if (!babelUsed) {
        require('babel-register')({
            presets: ['es2015', 'flow'],
            ignore: /node_modules(?!\/(haul|react-native))/,
            retainLines: true,
            sourceMaps: 'inline',
        });
        require('babel-polyfill');
        babelUsed = true;
    }
};
var createBaseConfig = function (builder, dev, options) {
    var stack = builder.stack;
    var baseConfig = {
        name: builder.name,
        devtool: dev ? '#cheap-module-source-map' : '#source-map',
        module: {
            rules: [
                {
                    test: /\.jsx?$/,
                    exclude: (stack.hasAny('react-native')) ?
                        /node_modules\/(?!react-native|@expo|expo|lottie-react-native|haul|pretty-format|react-navigation)$/ :
                        /node_modules/,
                    use: [].concat(options.persistGraphQL ?
                        ['persistgraphql-webpack-plugin/js-loader'] :
                        []),
                },
                {
                    test: /\.graphqls/,
                    use: 'raw-loader',
                },
                {
                    test: /\.(graphql|gql)$/,
                    exclude: /node_modules/,
                    use: ['graphql-tag/loader'].concat(options.persistGraphQL ?
                        ['persistgraphql-webpack-plugin/graphql-loader'] :
                        []),
                },
            ],
        },
        resolve: {
            extensions: stack.hasAny('server') ?
                [".web.js", ".web.jsx", '.js', '.jsx'] :
                ["." + stack.platform + ".js", "." + stack.platform + ".jsx", '.native.js', '.native.jsx', '.js', '.jsx'],
            modules: [path.join(process.cwd(), 'node_modules'), 'node_modules'],
        },
        watchOptions: {
            ignored: /build/,
        },
        bail: !dev,
    };
    if (stack.hasAny(['web', 'server'])) {
        baseConfig.resolve.alias = {
            'react-native': 'react-native-web',
        };
        baseConfig.module.rules = baseConfig.module.rules.concat([
            {
                test: /\.(png|ico|jpg|xml)$/,
                use: 'url-loader?name=[hash].[ext]&limit=10000',
            },
            {
                test: /\.woff(2)?(\?v=[0-9]\.[0-9]\.[0-9])?$/,
                use: 'url-loader?name=./assets/[hash].[ext]&limit=10000',
            },
            {
                test: /\.(ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
                use: 'file-loader?name=./assets/[hash].[ext]',
            },
        ]);
    }
    else if (stack.hasAny('react-native')) {
        baseConfig.module.rules = baseConfig.module.rules.concat([
            {
                test: mobileAssetTest,
                use: {
                    loader: require.resolve('./react-native/assetLoader'),
                    query: { platform: stack.platform, root: path.resolve('.'), bundle: false },
                },
            },
        ]);
    }
    return baseConfig;
};
var persistPlugins;
var ExtractTextPlugin;
var createPlugins = function (builder, builders, dev, options) {
    var stack = builder.stack;
    var webpack = requireModule_1.default('webpack');
    var buildNodeEnv = dev ? (stack.hasAny('test') ? 'test' : 'development') : 'production';
    if (!persistPlugins) {
        var PersistGraphQLPlugin = requireModule_1.default('persistgraphql-webpack-plugin');
        var moduleName = path.resolve('node_modules/persisted_queries.json');
        if (options.persistGraphQL) {
            var clientPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName,
                filename: 'extracted_queries.json', addTypename: true });
            var serverPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName,
                provider: clientPersistPlugin });
            persistPlugins = { client: clientPersistPlugin, server: serverPersistPlugin };
        }
        else {
            var clientPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName });
            var serverPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName });
            persistPlugins = { client: clientPersistPlugin, server: serverPersistPlugin };
        }
    }
    var plugins = [];
    if (dev) {
        plugins.push(new webpack.NamedModulesPlugin());
    }
    else {
        plugins.push(new webpack.optimize.UglifyJsPlugin({ minimize: true }));
        plugins.push(new webpack.LoaderOptionsPlugin({ minimize: true }));
        plugins.push(new webpack.optimize.ModuleConcatenationPlugin());
    }
    var backendUrl = options.backendUrl.replace('{ip}', ip.address());
    if (stack.hasAny('server')) {
        plugins = plugins.concat([
            new webpack.BannerPlugin({
                banner: 'require("source-map-support").install();',
                raw: true, entryOnly: false,
            }),
            new webpack.DefinePlugin({
                __CLIENT__: false, __SERVER__: true, __SSR__: options.ssr,
                __DEV__: dev, 'process.env.NODE_ENV': "\"" + buildNodeEnv + "\"",
                __PERSIST_GQL__: options.persistGraphQL,
                __BACKEND_URL__: "\"" + backendUrl + "\"",
            }),
            persistPlugins.server,
        ]);
    }
    else {
        plugins = plugins.concat([
            new webpack.DefinePlugin({
                __CLIENT__: true, __SERVER__: false, __SSR__: options.ssr,
                __DEV__: dev, 'process.env.NODE_ENV': "\"" + buildNodeEnv + "\"",
                __PERSIST_GQL__: options.persistGraphQL,
                __BACKEND_URL__: (stack.platform !== 'web' ||
                    url.parse(backendUrl).hostname !== 'localhost') ? "\"" + backendUrl + "\"" : false,
            }),
            persistPlugins.client,
        ]);
        if (stack.hasAny('web')) {
            var ManifestPlugin = requireModule_1.default('webpack-manifest-plugin');
            plugins.push(new ManifestPlugin({
                fileName: 'assets.json',
            }));
            var hasServer = false;
            for (var name in builders) {
                if (builders[name].stack.hasAny('server')) {
                    hasServer = true;
                    break;
                }
            }
            if (!hasServer) {
                var HtmlWebpackPlugin = requireModule_1.default('html-webpack-plugin');
                plugins.push(new HtmlWebpackPlugin({
                    template: 'tools/html-plugin-template.ejs',
                    inject: 'body',
                }));
            }
            if (!dev) {
                ExtractTextPlugin = requireModule_1.default('extract-text-webpack-plugin');
                plugins.push(new ExtractTextPlugin({ filename: '[name].[contenthash].css', allChunks: true }));
                plugins.push(new webpack.optimize.CommonsChunkPlugin({
                    name: 'vendor',
                    filename: '[name].[hash].js',
                    minChunks: function (module) {
                        return module.resource && module.resource.indexOf(path.resolve('./node_modules')) === 0;
                    },
                }));
            }
        }
        else if (stack.hasAny('react-native')) {
            plugins.push(new webpack.SourceMapDevToolPlugin({
                test: /\.(js|jsx|css|bundle)($|\?)/i,
                filename: '[file].map',
            }));
        }
    }
    if (stack.hasAny('dll')) {
        var name = "vendor_" + builder.parentName;
        plugins = [
            new webpack.DefinePlugin({
                __DEV__: dev, 'process.env.NODE_ENV': "\"" + buildNodeEnv + "\"",
            }),
            new webpack.DllPlugin({
                name: name,
                path: path.join(options.dllBuildDir, name + "_dll.json"),
            }),
        ];
    }
    return plugins;
};
var getDepsForNode = function (builder, depPlatforms) {
    var deps = [];
    for (var _i = 0, _a = Object.keys(pkg.dependencies); _i < _a.length; _i++) {
        var key = _a[_i];
        var val = depPlatforms[key];
        if (!val || (val.constructor === Array && val.indexOf(builder.parentName) >= 0) || val === builder.parentName) {
            deps.push(key);
        }
    }
    if (builder.stack.hasAny('react-native')) {
        deps = deps.concat(require.resolve('./react-native/react-native-polyfill.js'));
    }
    return deps;
};
var createConfig = function (builder, builders, dev, opts, depPlatforms) {
    var stack = builder.stack;
    var options = __assign({}, opts);
    if (stack.hasAny('test')) {
        options.ssr = false;
        options.persistGraphQL = false;
    }
    var baseDevServerConfig = {
        hot: true,
        contentBase: '/',
        publicPath: '/',
        headers: { 'Access-Control-Allow-Origin': '*' },
        quiet: false,
        noInfo: true,
        stats: { colors: true, chunkModules: false },
    };
    var config;
    if (stack.hasAny('react-native')) {
        useBabel();
    }
    var plugins = createPlugins(builder, builders, dev, options);
    if (stack.hasAny('server')) {
        var nodeExternals = requireModule_1.default('webpack-node-externals');
        var nodeExternalsFn_1 = nodeExternals({
            whitelist: [/(^webpack|^react-native)/],
        });
        config = __assign({}, createBaseConfig(builder, dev, options), { entry: {
                index: [
                    'babel-polyfill',
                    './src/server/index.js',
                ],
            }, target: 'node', node: {
                __dirname: true,
                __filename: true,
            }, externals: function (context, request, callback) {
                return nodeExternalsFn_1(context, request, function () {
                    if (request.indexOf('react-native') >= 0) {
                        return callback(null, 'commonjs ' + request + '-web');
                    }
                    else {
                        return callback.apply(this, arguments);
                    }
                });
            }, output: {
                devtoolModuleFilenameTemplate: dev ? '../../[resource-path]' : undefined,
                devtoolFallbackModuleFilenameTemplate: dev ? '../../[resource-path];[hash]' : undefined,
                filename: '[name].js',
                sourceMapFilename: '[name].[chunkhash].js.map',
                path: path.resolve(options.backendBuildDir),
                publicPath: '/',
            }, plugins: plugins });
    }
    else if (stack.hasAny('web')) {
        var backendUrl = options.backendUrl.replace('{ip}', ip.address());
        var _a = url.parse(backendUrl), protocol = _a.protocol, host = _a.host;
        var backendBaseUrl = protocol + '//' + host;
        config = __assign({}, createBaseConfig(builder, dev, options), { entry: {
                index: [
                    'babel-polyfill',
                    './src/client/index.jsx',
                ],
            }, output: {
                filename: '[name].[hash].js',
                path: path.resolve(path.join(options.frontendBuildDir, 'web')),
                publicPath: '/',
            }, plugins: plugins, devServer: __assign({}, baseDevServerConfig, { port: options.webpackDevPort, proxy: {
                    '!/*.hot-update.{json,js}': {
                        target: backendBaseUrl,
                        logLevel: 'info',
                    },
                } }) });
    }
    else if (stack.hasAny('react-native')) {
        var AssetResolver_1 = requireModule_1.default('haul/src/resolvers/AssetResolver');
        var HasteResolver = requireModule_1.default('haul/src/resolvers/HasteResolver');
        config = __assign({}, createBaseConfig(builder, dev, options), { entry: {
                index: [
                    require.resolve('./react-native/react-native-polyfill.js'),
                    './src/mobile/index.js',
                ],
            }, output: {
                filename: "index.mobile.bundle",
                publicPath: '/',
                path: path.resolve(path.join(options.frontendBuildDir, builder.name)),
            }, devServer: __assign({}, baseDevServerConfig, { hot: false, port: stack.hasAny('android') ? 3010 : 3020 }), plugins: plugins });
        config.resolve.plugins = [
            new HasteResolver({
                directories: [path.resolve('node_modules/react-native')],
            }),
            new AssetResolver_1({ platform: stack.platform, test: mobileAssetTest }),
        ];
        config.resolve.mainFields = ['react-native', 'browser', 'main'];
    }
    else {
        throw new Error("Unknown platform target: " + stack.platform);
    }
    if (stack.hasAny('dll')) {
        var name = "vendor_" + builder.parentName;
        config = __assign({}, config, { devtool: '#cheap-module-source-map', entry: {
                vendor: getDepsForNode(builder, depPlatforms),
            }, output: {
                filename: name + ".[hash]_dll.js",
                path: path.resolve(options.dllBuildDir),
                library: name,
            } });
    }
    return config;
};
exports.default = createConfig;
//# sourceMappingURL=generator.js.map