"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var merge = require("webpack-merge");
var configRc_1 = require("./configRc");
var generator_1 = require("./generator");
var Stack_1 = require("./Stack");
var requireModule_1 = require("./requireModule");
var Spin_1 = require("./Spin");
var CssProcessorPlugin_1 = require("./plugins/CssProcessorPlugin");
var ES6Plugin_1 = require("./plugins/ES6Plugin");
var WEBPACK_OVERRIDES_NAME = 'webpack.overrides.js';
var createConfig = function (cmd) {
    var builders = {};
    var plugins = [
        new CssProcessorPlugin_1.default(),
        new ES6Plugin_1.default()
    ];
    var config = new configRc_1.default(plugins);
    var options = config.options;
    var spin = new Spin_1.default(process.argv, config.builders, config.options);
    try {
        var _loop_1 = function (name) {
            var builder = config.builders[name];
            var stack = builder.stack;
            if (builder.roles.indexOf(cmd) < 0)
                return "continue";
            var overrides = void 0;
            var overridesConfig = options.overridesConfig || WEBPACK_OVERRIDES_NAME;
            if (fs.existsSync(overridesConfig)) {
                overrides = requireModule_1.default('./' + overridesConfig);
            }
            else {
                overrides = {};
            }
            builders[name] = __assign({}, builder, { config: generator_1.default(builder, config.builders, spin.dev, options) });
            config.plugins.forEach(function (plugin) {
                builders[name].config = merge(builders[name].config, plugin.configure(builder, spin));
            });
            if (overrides[name]) {
                builders[name].config = merge(builders[name].config, overrides[name]);
            }
            if (options.webpackDll && !stack.hasAny('server')) {
                var dllNode = __assign({}, builder);
                var dllNodeName = builder.name + 'Dll';
                dllNode.parentName = builder.name;
                dllNode.name = dllNodeName;
                dllNode.stack = new Stack_1.default(dllNode.stack.technologies, 'dll');
                builders[name].dllConfig = generator_1.default(dllNode, config.builders, spin.dev, options, overrides.dependencyPlatforms || {});
                config.plugins.forEach(function (plugin) {
                    builders[name].dllConfig = merge(builders[name].dllConfig, plugin.configure(builder, spin));
                });
                if (overrides[dllNodeName]) {
                    builders[name].dllConfig = merge(builders[name].dllConfig, overrides[dllNodeName]);
                }
            }
        };
        for (var name in config.builders) {
            _loop_1(name);
        }
    }
    catch (e) {
        console.error(e.stack);
    }
    return { builders: builders, options: options };
};
exports.default = createConfig;
//# sourceMappingURL=spin.config.js.map